@@ -1,5 +1,5 @@
 package Chemistry::Mol;
-$VERSION = '0.06';
+$VERSION = '0.10';
 
 =head1 NAME
 
@@ -9,9 +9,10 @@ Chemistry::Mol - Molecule object toolkit
 
     use Chemistry::Mol;
 
-    $mol = new Chemistry::Mol(id => "mol_id");
-    $mol->add_atom($atom1, $atom2);
-    $mol->add_bond($bond);
+    $mol = Chemistry::Mol->new(id => "mol_id", name => "my molecule");
+    $c = $mol->new_atom(symbol => "C", coords => [0,0,0]); 
+    $o = $mol->new_atom(symbol => "O", coords => [0,0,1.23]); 
+    $mol->new_bond(atoms => [$c, $o], order => 3);
 
     print $mol->print;
 
@@ -30,24 +31,16 @@ internal coming soon).
 
 =cut
 
+use 5.006001;
 use strict;
 use Chemistry::Atom;
 use Chemistry::Bond;
 use Carp;
-use base qw(Exporter Chemistry::Obj);
+use base qw(Chemistry::Obj);
 
 
-our @EXPORT = qw();
-our @EXPORT_OK = qw( read_mol );
-
-our %EXPORT_TAGS = (
-   all  => [@EXPORT, @EXPORT_OK]
-);
-
-use overload '""' => "stringify";
-
-our %FILE_FORMATS = ();
-my $N = 0;
+my %FILE_FORMATS = ();
+my $N = 0; # atom ID counter
 
 =head1 METHODS
 
@@ -78,7 +71,7 @@ sub new {
 	atoms => [], 
 	bonds => [], 
 	name => "",
-    }, $class;
+    }, ref $class || $class;
     $self->$_($args{$_}) for (keys %args);
     return $self;
 }
@@ -87,6 +80,10 @@ sub nextID {
     "mol".++$N; 
 }
 
+sub reset_id {
+    $N = 0; 
+}
+
 =item $mol->add_atom($atom, ...)
 
 Add one or more Atom objects to the molecule. Returns the last atom added.
@@ -158,14 +155,16 @@ sub by_id {
 
 =item $mol->atoms($n1, ...)
 
-Returns the atoms with the given indices, or all by default.
+Returns the atoms with the given indices, or all by default. 
+Indices start from one, not from zero.
 
 =cut
 
 sub atoms {
     my $self = shift;
-    if (@_) {
-        @{$self->{atoms}}[@_];
+    my @ats = map {$_ - 1} @_;
+    if (@ats) {
+        @{$self->{atoms}}[@ats];
     } else {
         @{$self->{atoms}};
     }
@@ -181,37 +180,62 @@ expression).
 sub atoms_by_name {
     my $self = shift;
     my $re = qr/^$_[0]$/;
+    no warnings;
     my @ret = grep {$_->name =~ $re} $self->atoms;
-    wantarray ? @ret : @ret[0];
+    #my ($re) = @_; # 5.004 hack
+    #my @ret = grep {defined $_->name and $_->name =~ /$re/o} $self->atoms;
+    wantarray ? @ret : $ret[0];
 }
 
+=for comment
+sub select_atoms {
+    my $self = shift;
+    my %opts = @_;
+    my @a = $self->atoms;
+    for my $opt (keys %opts) {
+        my $re = qr/^$opts{$opt}$/;
+        @a = grep {$_->$opt =~ $re} @a;
+    }
+    @a;
+}
+
+=cut
+
 =item $mol->bonds($n1, ...)
 
 Returns the bonds with the given indices, or all by default.
+Indices start from one, not from zero.
 
 =cut
 
 sub bonds {
     my $self = shift;
-    if (@_) {
-        @{$self->{bonds}}[@_];
+    my @bonds = map {$_ - 1} @_;
+    if (@bonds) {
+        @{$self->{bonds}}[@bonds];
     } else {
         @{$self->{bonds}};
     }
 }
 
-=item $mol->print
+=item $mol->print(option => value...)
 
-Convert the molecule to a string representation. 
+Convert the molecule to a string representation. If no options are given, 
+a default YAML-like format is used (this may change in the future). Otherwise,
+the format should be specified by using the C<format> option.
 
 =cut
 
 sub print {
     my $self = shift;
+    my (%opts) = @_;
     my $ret;
-    my ($a, $b);
     local $" = ""; #"
 
+    if ($opts{format}) {
+        return $self->formats($opts{format})->write_string($self, %opts);
+    }
+    # else use default printout 
     $ret = <<END;
 $self->{id}:
     name: $self->{name}
@@ -219,99 +243,144 @@ END
     $ret .= "    attr:\n";
     $ret .= $self->print_attr(2);
     $ret .= "    atoms:\n";
-    for $a (@{$self->{atoms}}) { $ret .= $a->print(2) }
+    for my $a (@{$self->{atoms}}) { $ret .= $a->print(2) }
     $ret .= "    bonds:\n";
-    for $b (@{$self->{bonds}}) { $ret .= $b->print(2) }
+    for my $b (@{$self->{bonds}}) { $ret .= $b->print(2) }
     $ret;
 }
 
-=begin comment
+=item $mol->parse($string, option => value...)
 
-=item read_mol($fname, [$type])
+Parse the molecule encoded in $string. The format should be specified
+with the the C<format> option; otherwise, it will be guessed.
 
-Read a file returning a list of Mol objects, or undef if there
+=cut
+
+sub parse {
+    my $self = shift;
+    my $s = shift;
+    my %opts = (mol_class => $self, @_);
+
+    if ($opts{format}) {
+        return $self->formats($opts{format})->parse_string($s, %opts);
+    } else {
+        croak "Parse does not support autodetection yet.",
+            "Please specify a format.";
+    }
+    undef;
+}
+
+=item Chemistry::Mol->read($fname, option => value ...)
+
+Read a file and return a list of Mol objects, or croaks if there
 was a problem. The type of file will be guessed if not
-specified.
+specified via the C<format> option.
 
 Note that only registered file readers will be used. Readers may
 be registered using register_type(); modules that include readers
 (such as Chemistry::File::PDB) usually register them automatically.
 
-This function may be exported.
+=cut
+
+sub read_mol { # for backwards compatibility
+    my ($fname, $type) = shift;
+    Chemistry::Mol->read($fname, format => $type);
+}
+
+sub read {
+    my $self = shift;
+    my $fname = shift;
+    my %opts = (mol_class => $self, @_);
+
+    if ($opts{format}) {
+        return $self->formats($opts{format})->parse_file($fname, %opts);
+    } else { # guess format
+        for my $type ($self->formats) {
+            if ($self->formats($type)->file_is($fname)) {
+                return $self->formats($type)->parse_file($fname, %opts);
+            }
+        }
+    }
+    croak "Couldn't guess format of file '$fname'";
+}
+
+=item $mol->write($fname, option => value ...)
+
+Write a molecule file, or croak if there
+was a problem. The type of file will be guessed if not
+specified via the C<format> option.
+
+Note that only registered file formats will be used. 
 
 =cut
 
-sub read_mol {
-    my $fname = shift;
-    my $type = shift;
+sub write {
+    my ($self, $fname, %opts) = (@_);
 
-    if ($type) {
-	return $FILE_FORMATS{$type}->{read}($fname);
-    } else {
-	#print "No type specified...\n";
-	for $type (keys %FILE_FORMATS) {
-	    #print "is file $type?\n";
-	    if ($FILE_FORMATS{$type}->{is}($fname)){
-		#print "file is $type!\n";
-		return $FILE_FORMATS{$type}->{read}($fname);
+    if ($opts{format}) {
+        return $self->formats($opts{format})->write_file(@_);
+    } else { # guess format
+	for my $type ($self->formats) {
+            if ($self->formats($type)->name_is($fname)) {
+                return $self->formats($type)->write_file(@_);
 	    }
 	}
     }
-    undef;
+    croak "Couldn't guess format of file '$fname'";
 }
 
-=item register_type($name, sub_id => \&sub,... )
+=item Chemistry::Mol->register_format($name, $ref)
 
 Register a file type. The identifier $name must be unique.
-To register a file type, you need to provide references to 
-at least some of the following functions, identified by their 
-respective sub_id's.
+$ref is either a class name (a package) or an object that complies
+with the L<Chemistry::File> interface (e.g., a subclass of Chemistry::File).
 
-=over 4
-
-=item is
-
-A reference to a function that receives a filename and returns 1 if the
-file is of the specified type.
-
-=item read 
-
-A reference to a function that receives a filename and returns the list
-of Mols contained in the file.
-
-=item write 
-
-A reference to a function that receives a filename and a list of
-molecules and writes the molecules to the file.
-
-=item parse
-
-A reference to a function that receives a string and returns the list
-of Mols contained in the string.
-
-=item print
-
-A reference to a function that receives a list of
-molecules and returns a string.
-
-=back
-
-=end comment 
-
-=back
+The typical user doesn't have to care about this function. It is used
+automatically by molecule file I/O modules.
 
 =cut
 
-sub register_type {
-    my $type = shift;
-    $FILE_FORMATS{$type} = {@_};
+sub register_format {
+    my $class = shift;
+    my %opts = @_;
+    $FILE_FORMATS{$_} = $opts{$_} for keys %opts;
+}
+
+=item Chemistry::Mol->formats
+
+Returns a list of the file formats that have been installed by
+register_type()
+
+=cut
+
+sub formats {
+    my $self = shift;
+    if (@_) {
+        my ($type) = @_;
+        my $file_class = $FILE_FORMATS{$type};
+        unless ($file_class) {
+            croak "No class installed for type '$type'";
+        }
+        return $file_class;
+    } else {
+        return sort keys %FILE_FORMATS;
+    }
 }
 
 1;
 
+=back
+
+=head1 BUGS
+
+Blatant memory leaks. Due to the use of circular references, Perl's current
+garbage collector never cleans up molecule, atom, and bond objects. A future
+version should address this.
+
 =head1 SEE ALSO
 
-L<Chemistry::Atom>, L<Chemistry::Bond>, L<Chemistry::File>
+L<Chemistry::Atom>, L<Chemistry::Bond>, L<Chemistry::File>,
+L<Chemistry::Tutorial>
 
 =head1 AUTHOR
 
